/*
 * Copyright 2006, Thomas Maier-Komor
 * LICENSE: GPLv2, see file COPYING for details
 */

#ifndef SUPPORT_H
#define SUPPORT_H

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <string.h>


static inline char *
Kb2str(double v, char *s)
{
	const char *dim = "kMGT", *f;

	while (v > 10000) {
		v /= 1024.0;
		++dim;
	}
	if (v < 0) {
		f = " ??? ";
	} else if (rint(v) >= 100)
		f = "%4.0f%c";
	else
		f = "%4.1f%c";
	sprintf(s,f,v,*dim);
	return s + 5;
}


static inline char *
kb2str(double v)
{
	static char str[8];
	const char *dim = "kMGT", *f;

	while (v > 10000) {
		v /= 1024.0;
		++dim;
	}
	if (v < 0) {
		f = " ??? ";
	} else if (rint(v) >= 100)
		f = "%4.0f%c";
	else
		f = "%4.1f%c";
	sprintf(str,f,v,*dim);
	return str;
}


static inline char *
Per2str(double d, char *s)
{
	const char *f;
	assert(d <= 1E2);
	if (d < 0) {
		f = "n/a ";
	} else if (d < 2E-2) {
		f = "%4.0f";
	} else if (d < 9.995) {
		f = "%4.2f";
	} else if (d < 99.95) {
		f = "%4.1f";
	} else {
		f = "%4.0f";
		d = rint(d);
	}
	sprintf(s,f,d);
	return s + 4;
}

static inline char *
per2str(double d)
{
	static char str[8];
	const char *f;
	assert(d <= 1E2);
	if (d < 0) {
		f = "n/a ";
	} else if (d < 2E-2) {
		f = "%4.0f";
	} else if (d < 9.995) {
		f = "%4.2f";
	} else if (d < 99.95) {
		f = "%4.1f";
	} else {
		f = "%4.0f";
		d = rint(d);
	}
	sprintf(str,f,d);
	return str;
}


static inline char *
Dbl2str(double d, char *s)
{
	const char *dim = " kMG", *f;
	double r = d;

	while (r >= 1E4) {
		r /= 1E3;
		++dim;
	}
	if (d < 0)
		f = "unav.";
	else if (r < 2E-2)
		f = "%4.0f%c";
	else if (r < 9.995)
		f = "%4.2f%c";
	else if (r < 1E2)
		f = "%4.1f%c";
	else
		f = "%4.0f%c";
	sprintf(s,f,r,*dim);
	return s + 5;
}


static inline char *
dbl2str(double d)
{
	static char str[8];
	const char *dim = " kMG", *f;
	double r = d;

	while (r >= 1E4) {
		r /= 1E3;
		++dim;
	}
	if (d < 0)
		f = "unav.";
	else if (r < 2E-2)
		f = "%4.0f%c";
	else if (r < 9.995)
		f = "%4.2f%c";
	else if (r < 1E2)
		f = "%4.1f%c";
	else
		f = "%4.0f%c";
	sprintf(str,f,r,*dim);
	return str;
}


#endif
